PROGRAM CONRED44;
{$F+}

{***********************************************************
"CONVERTIDOR 6 PULSOS DOS VIAS TRABAJANDO EN MODO INVERSOR

Autores: JRomero - Arodri
Fecha: 16-02-98

		 La simulacin corresponde a un CONVERTIDOR 6 PULSOS DOS VIAS
		 A TIRISTORES CON INDUCTANCIA SERIE Y QUE TRABAJA EN MODO INVERSOR.

		 y se observarn los siguientes parmetros:

		 * tensines entre fases de entrada, tension  de salida
		 * corriente de carga.
		 * corriente de alimentacin en la fase R junto a su tension de fase correspondiente
		 * corriente de los TIRISTORES 3 junto con su tension en bornes
		 * corriente de los TIRISTORES 1.
**********************************************************}


{***********************************************************
" CONVERTIDOR TRABAJANDO COMO INVERSOR CON INDUCTANCIA DE LINEA"
Autores: JRomero - Arodri
Fecha: 16-02-98

		 y se observarn los siguientes parmetros:

		 * tensin de entrada, contrastada con la de salida
		 * corriente de carga.
		 * corriente de alimentacin en la fase R
		 * corriente de los TIRISTORES
**********************************************************}

USES
wincrt,xmatdefs, wingraph,dibus,FilUSR, UCntrl, Simul4, traxp, variable,
lib_basi, libep02, libep, libdig01;

CONST w=PI*100;           {frecuencia de red=50 hz}
		periodo=20e-3;      {periodo=2*PI/w}
		Npuntos=1024;       {nmero de puntos de muestreo}
		sexrad=PI/180;      {constante de conversin de sexagesimal a rad}

var	Alpha,  L_conmu,       {.Para controlar variables.}

	Intensidad, grados:NReal; {variables para visualizar en pantalla}
	tension,ir, beta, U, vector_u, alpha_viejo, u_viejo, Gama, t_falla:NReal;

	primer_ciclo, falla, falla_ventana:Boolean;
	grad, enY1, enY2:Integer;

{----------------------------------------------------------------------}

FUNCTION disparo(k:integer): boolean;
var  vref:NREAL;
BEGIN
		vref:=-cos(alpha*sexrad);
		disparo:=(vref<=sin(w*t-2*PI/3-(k-1)*PI/3)) and (cos(w*t-2*PI/3-(k-1)*PI/3)>=0);

IF	 (t<0.001) and (primer_ciclo=TRUE) then  {Condiciones iniciales}
		begin
		if (K=3) or (K=4) then disparo:=TRUE;
		end else primer_ciclo:=FALSE;
END;

{----------------------------------------------------------------------}
{*********************************************************
A continuacin los procedimientos Cirt1, Cirt2 y Funcsr de
definicin del circuito.
*********************************************************}


PROCEDURE CIRT1;
BEGIN
	  vg(1,6,7, tension);            {Tension de red R}
	  vg(2,6,8, tension);            {Tension de red S}
	  vg(3,6,9, tension);     		  {Tension de red T}

	  L(7,16, L_CONMU);   	     {L lnea}
	  L(8,2,  L_CONMU);	        {L lnea}
	  L(9,3,  L_CONMU);	        {L lnea}
	  R(7,16,1000);			{r para bobina del primario}
	  R(8,2,1000);				{r para bobina del primario}
	  R(9,3,1000);				{r para bobina del primario}

	  r(16,1,0.01);         {R para medir la intensidad de fase R}

	  r(4,10, 0.01);				{r para medir i de tiristores}
	  r(4,11, 0.01);				{r para medir i de tiristores}
	  r(4,12, 0.01);				{r para medir i de tiristores}

	  r(0,13, 0.01);				{r para medir i de tiristores}
	  r(0,14, 0.01);				{r para medir i de tiristores}
	  r(0,15, 0.01);				{r para medir i de tiristores}

	  ig(0,4,5, 1);            {fuente de I=Cte. a la salida}
	  R(5,0, 0.1);             {R para medir la intensidad de salida}
END;

{---------------------------------------------------------------------}

PROCEDURE CIRT2;
BEGIN
	  th(1,10,disparo(1));     {T1}
	  th(15,3,disparo(2));     {T2}
	  th(2,11,disparo(3));     {T3}
	  th(13,1,disparo(4));     {T4}
	  th(3,12,disparo(5));     {T5}
	  th(14,2,disparo(6));     {T6}
END;

{---------------------------------------------------------------------}

PROCEDURE FUNCSR(t:NReal);
BEGIN
rf[1]:=sin(w*t);        	{R}
rf[2]:=sin(w*t -2*PI/3);	{S}
rf[3]:=sin(w*t -4*PI/3);	{T}
END;

{---------------------------------------------------------------------}

{********************************************************
A continuacin los procedimientos de inicio, fin  y Pesca
necesarios para Simeep
*********************************************************}

PROCEDURE INICIO;
BEGIN
nn:=16;			{nmero de nodos del circuito sin incluir 0 (Tierra)}
nindep:=4;		{nmero de fuentes independientes}
tinicial:=0e-3;   {tiempo incial de cada simulacin}
tfinal:=20e-3; {tiempo final de cada simulacin}

{Valores iniciales de variables de estado.
 Estas se definen en el orden en que aparecen las inductancias
 y Capacitancias en Cirt1}

x0[1]:= -0.991557;		{corresponde a Lconm R}
x0[2]:=  0.965521;		{corresponde a Lconm S}
x0[3]:=  0.026036;		{corresponde a Lconm T}

{valores de precisin de Simeep}
dtmax0:=10e-6;
dtmin:=dtmax0/100;
PasoDivisor:=6;
TmaxTmin:=1e3;

{indicamos a TRAXP los tiempos que debe graficar}
traxp.tinicial:=tinicial;
traxp.tfinal:=tfinal;

{En esta seccin, definimos el nmero de pantallas, y los colores,
 en nmero de grillas inciales, y si tenemos alguna superpocicin
 de seales en una misma ventana}

gridx:=12;	{las divisiones en X son 12}
gridy:=8;	{las diviciones en Y son 8}
subplot(1,2); {matriz de pantallas de 1 por 1}

{definiciones de visualizacin para la pantalla 0,est asociada al canal 0}
definaY(0,-600, 150);
titulo('(Azul):Ud.  vRS         vRT         vST         vSR          vTR          vTS');
xlabel(' 30/DIV');
ylabel('150 V/DIV');
linea0;
grid;
{asociamos otros canales a la pantalla 0}
superponga(2,0);
superponga(3,0);
superponga(4,0);
superponga(5,0);
superponga(6,0);
superponga(7,0);
{definimos otros colores para algunos de los canales}
definacolor(0, azul);
definacolor(2, negro);
definacolor(3, violeta);
definacolor(4, 12);
definacolor(5, 6);
definacolor(6, 14);
definacolor(7, 8);

{definiciones de visualizacin para la pantalla 1, est asociada al canal 1}
definaY(1,-140, 20);
titulo(' (Morado): inten. tiristores.  (Amarillo): iR.   (Rojo): uR.   (Verde): uT3.');
xlabel(' 30/DIV');
ylabel('     T3          T1       iR,vR');
linea0;
grid;
{asociamos otros canales a la pantalla 1}
superponga(8, 1);
superponga(9, 1);
superponga(10,1);
superponga(11,1);
 definacolor(1,  amarillo);
 definacolor(8,  12);
 definacolor(9,  5);
 definacolor(10, 5);
 definacolor(11, 10);
END;

{---------------------------------------------------------------------}

PROCEDURE PESCA;
var   coseno, coseno_absoluto:NReal; k, enX:integer;
BEGIN
grados:=18000*t; {....Para ver en el display el tiempo y los grados.}
if grados>=360 then grados:=grados-360*Int(t/0.02);


if (falla_ventana=true) and (t>t_falla+0.002) then
begin
PONER_VENTANA(100,100,300,200,'  FALLA POR CONMUTACION',
													 '',
													 'Disminuya el valor de alpha ',
													 ' por debajo de 120 grados.');
falla_ventana:=false;
end;

			  {Calculo el tiempo de conmutacion uk}
coseno:=cos(alpha*PI/180)-w*L_conmu*2*intensidad/ (SQRT(3)*tension);
if (coseno>-0.000001) AND (coseno<0.000001) then coseno:=0.000001;
coseno_absoluto:=ABS(coseno);
vector_U:=ArcTan(SQRT(ABS(1-SQR(coseno_absoluto)))/coseno_absoluto);

if coseno<0 then	vector_U:=PI-vector_U; {aqui tengo u }

vector_u:=vector_u*180/pi;
U:=vector_U-alpha;

						{compruebo si alpha+u ha superado los 180}
if (alpha>alpha_viejo) and (u<=u_viejo) then
													 begin
													 Sound(500,100);
													 vector_u:=360-vector_u;
													 u:=vector_u-alpha;
													 gama:=180-alpha-U;
													 beta:=180-alpha;
													 falla:=true;
													 falla_ventana:=true;
													 t_falla:=t;
													 end

													 else
													 begin
													  if falla=false then
													  begin
													  gama:=180-alpha-U;
													  beta:=180-alpha;
													  end;
													 end;


if alpha<120 then falla:=false;

alpha_viejo:=alpha;  {restauro valores}
u_viejo:=u;


ir:=v[16]-v[1];        {calculo la intensidad de fase R}
Intensidad:=v[5]/0.1;  {calculo la intensidad de salida}


if NOT falla then
begin
	 {...Pinto anotaciones sobre las graficas...}

enX:=PUNTOX+24; {paso la ordenada x que pinta la onda a la variable enX por medio de la funcion puntox}


{ANOTAR_EVENTO(GRAD1,GRAD2,GRAD, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{alpha}ANOTAR_EVENTO(1,30,30+alpha+1,grados, enx,70,38,100,60,enY2, CHR(224));

{ANOTAR_EVENTO(GRAD1,GRAD2,GRAD, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{Uu}ANOTAR_EVENTO(2,30+alpha+1,30+alpha+U+1, grados,enx,100,97,enY2-22,97,enY2, 'U');

{ANOTAR_EVENTO(GRAD1,GRAD2,GRAD, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{Ui}ANOTAR_EVENTO(4,30+alpha+1,30+alpha+U+1, grados,enx,377,360,380,310,380, 'U');

{ANOTAR_EVENTO(GRAD1,GRAD2,GRAD, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{gamau}ANOTAR_EVENTO(5,30+alpha+U+1, 180+30, grados,enx,217,90,220,190,220, 'Gama');

{ANOTAR_EVENTO(GRAD1,GRAD2,GRAD, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{beta}ANOTAR_EVENTO(6,30+alpha+1, 180+30, grados,enx,80,77,enY2-66,77,200, 'Beta');

{ANOTAR_EVENTO(GRAD1,GRAD2,GRAD, ENX,       y,y1a,y1b,y2a,y2b,etiqueta)}
{gamai}ANOTAR_EVENTO(3,alpha-90+U+1, alpha-90+U+gama+1, grados,enx,373,370,433,370,433, 'Gama');
end;


{indicar el tiempo de simulacin a el graficador}
traxp.t:=t;

{.........................grafica 0.....................}

trazo(0,v[4]);              { TENSION EN LA CARGA R-L }
trazo(0,0);
enY1:=PUNTOY+22; {paso la ordenada y que pinta la onda a la variable enY por medio de la funcion puntox}

{RELLENAR_EVENTO(GRAD1,     GRAD2,    GRADOS,ENX,  y1,   y2,   color)}
RELLENAR_EVENTO(30+alpha+1, 30+alpha+U, grados,enx, enY1, enY2,    14);

trazo(2,v[7]-v[8]);              { TENSION de fase R-S }
enY2:=PUNTOY+22; {paso la ordenada y que pinta la onda a la variable enY por medio de la funcion puntox}

trazo(3,v[7]-v[9]);              { TENSION de fase R-T }
trazo(4,v[8]-v[9]);              { TENSION de fase S-T }
trazo(5,v[8]-v[7]);              { TENSION de fase S-R }
trazo(6,v[9]-v[7]);              { TENSION de fase T-R }
trazo(7,v[9]-v[8]);              { TENSION de fase T-S }

{.........................grafica 1.....................}
trazo(1, (v[16]-v[1])    *1000);              { i fase R }
trazo(1,0);
trazo(8, (v[7]-v[6])*0.08*0.321    );              { v fase R }

trazo(9,  (v[10]-v[4])  *4000-60);     { intensidad por T1 }
trazo(9,0-60);

trazo(10, (v[11]-v[4])  *4000-120);    { intensidad por T3 }
trazo(10,0-120);                       { tension en el  T3 }
if (grados>alpha-90+U+1)  and (grados<alpha-90+U+120) then
					trazo(11, (v[8]-v[9])*0.16*0.321-120)
					else

if (grados>alpha-90+U+120)  and (grados<alpha-90+U+120+119-u) then
					trazo(11, (v[8]-v[7])*0.16*0.321-120)  {tension en el T3}
					else
					trazo(11, -120);


	{borrado de la pantalla despus de cada simulacin}
	if t >= tfinal then
	begin
		for k:= 0 to 11 do
		begin
		borrecanal(k,0);
		desligue(k);
		borde;
		grid;
		linea0;
		end;
		t:=t-tfinal+tinicial;
	end;


END;


{---------------------------------------------------------------------}

PROCEDURE FIN;
BEGIN
	halt;
END;


{---------------------------------------------------------------------}

PROCEDURE INICIALIZACION_PARAMETROS;
BEGIN
{valores iniciales de los controles}
primer_ciclo:=TRUE;
tension:=311;
alpha_viejo:=300;
u:=1;
u_viejo:=0;
falla:=false;
END;


{----------------------------------------------------------------------}
{******************************PRINCIPAL*******************************}
{----------------------------------------------------------------------}
BEGIN
INICIALIZACION_PARAMETROS;
{Indicamos a Simeep donde estn las funciones que preparamos}
FilUSR.Cirt1:=Cirt1;
FilUSR.Cirt2:=Cirt2;
FilUSR.FuncsR:=FuncsR;
FilUSR.Inicio:=Inicio;
FilUSR.Pesca:=Pesca;
FilUSR.Fin:=Fin;

{Pantalla de controles}
UCntrl.init;

{------------------PARA VISUALIZAR TEXTO EN LA SIMULACION--------------}

UCntrl.insertdisplay(new(PCtrlVerTexto,init
(    0,   '    CONVERTIDOR ',   ' 6 PULSOS DOS VIAS',
	 ' EN MODO  INVERSOR ')));

UCntrl.insertdisplay(new(PCtrlVerTexto,init
(    1,   '     ',
			 '        CARGA',
			 '    FUENTE I=Cte. ')));

UCntrl.insert(new(PCtrlVerTexto,init
(    2,   '    ',
			 '    VISUALIZACION    ',
			 '    DE  VARIABLES ')));



UCntrl.insert(new(PCtrlVerTexto,init
(    7,   '    ',
			 '     ',
			 'INDUC.SERIE Y DISPARO ')));


{---------------PARA CONTROLAR VARIABLES EN LA SIMULACION--------------}

UCntrl.insert(new(PCtrlBotonVar,Init
{renglon, nombre,  unidad, decimales, min, max,   def, inc,    var}
(    8, ' L_CONMU',  'mH'    ,0,       1,  100,  50, 10, L_CONMU)));

UCntrl.insert(new(PCtrlBotonVar,Init
{renglon, nombre,  unidad, decimales, min, max,   def, inc,    var}
(    9, ' Disparo',  ''    ,0,       90,   180,  126,  2,   alpha)));


{-----------------PARA VER VARIABLES EN LA SIMULACION------------------}

UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon, nombre1, uni1, deci1,  var1 , nombre2, uni2, deci2, var2}
(    6,  CHR(225),  '',      1  ,  Beta,  CHR(224),  '',   1 , Alpha)));


UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon, nombre1, uni1, deci1,  var1 , nombre2, uni2, deci2, var2}
(    5, 'u',  '',      1  ,  U , 'Gama',  '',   1 , gama)));


{UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon, nombre1, uni1, deci1,  var1 , nombre2, uni2, deci2, var2}
{(    6, 'in R',  '',      3  ,   ir ,   'Uk',  '',   1 , valor)));
}
UCntrl.insertdisplay(new(PCtrlVer2Var,init
 {renglon,  nombre1,  uni1, deci1,  var1 , nombre2,  uni2, deci2, var2}
(    3,    'Tiempo',   'Sg',   3  ,  t ,  'Grados',  '',    0 ,Grados)));


UCntrl.insertdisplay(new(PCtrlVer2Var,init
{ renglon, nombre1, uni1, deci1,  var1 , nombre2, uni2, deci2, var2}
(    4, 'v_carga',  'V',     1  , v[4] , 'i_carga', 'A',   3 , intensidad)));
{----------------------------------------------------------------------}
{Inicio de Simulaciones}
PASEO;
END.
