program conred11;
{$F+}
{
***********************************************************
"Rectificador monofsico controlado"
Autores: JRomero - Arodri
Fecha: 18-04-98

		 La simulacin corresponde a un rectificador monofsico
		 controlado, y se obeservarn los siguientes parmetros:

		 * tensin de entrada, contrastada con la de salida
		  tomando como carga una resitencia de 10 ohms.
		 * corriente de alimentacin
		 * contenido de armnicos de la corriente de alimentacin,
		 muestreando un periodo antes de tfinal

		 Se podr afectar el ngulo de disparo mediante la tecla "a" y
		 el valor del mismo estar acotado entre 0 y 170 grados. El
		 recorrido se har a pasos de 2 grados.

**********************************************************
}

uses
wincrt, xmatdefs, FilUSR, UCntrl, Simul4, traxp, variable,
lib_basi, libep02, libep, libdig01, PlotMems, Memorias,dibus;

const w=PI*100;           {frecuencia de red=50 hz}
		sexrad=PI/180;      {constante de conversin de sexagesimal a rad}
		periodo=20e-3;      {periodo=2*PI/w}
		Npuntos=2048;		  {nmero de puntos de muestreo}
		grillay=8;          {divisiones de la grilla en y}
		grillax=12;			  {divisiones de la grilla en x}
		nper=1;				  {nmero de periodos a visualizar   (nper>=1)}

var   alpha:NReal;
		UD,U1:NReal;				{valor medio de tension a la salida}
		mem1:Tmemoria;      {objeto para clculo de espectro de potencia}

(*********************************************************************
  Funcin para el disparo del tiristor.
*********************************************************************)
{
El parmetro ALPHA es tomado como control del disparo del tiristor.
}

function disparo: boolean;
var taux:NREAL;
begin
	  {PULSO DE 200 uS equivalente a 0.0628 radianes de conduccin}
	  taux:=t;							{tiempo de simulacin}

	  {Reducir el tiempo a un periodo}
	  while taux>=periodo do
	  begin
			 taux:=taux-periodo;
	  end;

	  disparo:=false;
	  if ((w*taux)>=(alpha*sexrad)) and   		{angulo mayor que el de disparo}
		  ((w*taux)<=(alpha*sexrad)+0.0628)    {duracin del disparo en radianes}
	  then disparo:=true;

end;



(***********************************************************************
	Funciones de definicin del circuito

	Cirt1: componentes no booleanos
	Cirt2: componentes booleanos
***********************************************************************)

procedure cirt1;
begin
vg(1,0,1,311);								{generador de 220 Vrms}
r(2,0,10);									{carga de 10 ohm}
end;

procedure cirt2;
begin
th(1,2,disparo);							{Tiristor}
end;


(************************************************************************
	Definicin de la seal del generador VG[1]
************************************************************************)
procedure funcsr(t:NReal);
begin
rf[1]:=sin(w*t);
end;




(************************************************************************
	Procedimientos necesarios de Simeep:

	inicio:	inicializacin de simulaciones, def. de pantalla,etc
	pesca: 	captura e impresin en pantalla de variables observadas
	fin:     procedimiento de finalizacin

************************************************************************)

procedure inicio;
begin
	nn:=2;					{nmero de nodos, sin incluir tierra (0)}
	nindep:=1;     		{nmero de funciones definidas en funcsr = numero de generadores}
	tinicial:=0;			{tiempo inicial de la simulacin}
	tfinal:=nper*periodo;	{tiempo final de la simulacin}


	{parmetros de simulacin}
	dtmax0:=5e-6;
	dtmin:=dtmax0/100;
	PasoDivisor:=6;
	TmaxTmin:=1e3;

	{inicializacin de la unidad de grficos}
	traxp.tinicial:=tinicial;
	traxp.tfinal:=tfinal;

	{inicializacin del modo grfico}
	iniciegr;

	{grillas en pantalla}
	gridx:=grillax;
	gridy:=grillay;

	{matriz de visualizacin de 1 por 4}
	subplot(1,4);


	{primera pantalla}
	definaY(0,-400,100);
	titulo('Tensin de la fuente');
	xlabel('30/DIV');
	ylabel('100 V/DIV');
	linea0;
	grid;

	{segunda pantalla}
	definaY(1,-400,100);
	titulo('Tensin en la carga (negro) y comando de disparo (rojo/adimensional)');
	xlabel('30 /DIV');
	ylabel('100 V/DIV');
	linea0;
	grid;

	{TERCERA pantalla}
	definaY(2,-400,100);
	titulo('Tensin sobre el tiristor');
	xlabel('30 /DIV');
	ylabel('100 A/DIV');
	linea0;
	grid;

	{tercera pantalla}
	definaY(3,-10,2.5);
	titulo('Magnitud de armnicos en uD(t) (1 al 25) respecto al primero. ');

	{canal 4 y 5 en la misma pantalla que el canal 0}
	superponga(4,1);


	{**************************
	colores a mostrar por canal
	***************************}
	{PANTALLA 1}
	definacolor(0,azul);	{tensin del generador}

	{PANTALLA 2}
	definacolor(1,negro);	{tensin en la carga}
	definacolor(4,rojo);	{pulso de disparo, ADIMENSIONAL slo intuitivo}

	{PANTALLA 3}
	definacolor(2,azul);	{tensin sobre el tiristor}

	{PANTALLA 4}
	definacolor(3,rojo);	{armnicos de la tensin en la carga}
end;

{____________________________________________________________________}

procedure fin;
begin
TermineGr;
halt;
end;

{____________________________________________________________________}

procedure pesca;

var   k,enx:integer;
		grados:NReal;
begin

	delay(1);					{para que la simulacion sea mas lenta}
	traxp.t:=t;					{indicamos a traxp el tiempo de simulacin}


	{*etiquetas de grficas*}
	grados:=(w/sexrad)*t;
	enX:=PUNTOX+20; {paso la ordenada x que pinta la onda a la variable enX por medio de la funcion puntox}

	{ANOTAR_EVENTO(numeracin, GRAD1,GRAD2,GRAD, ENX, y,y1a,y1b,y2a,y2b,etiqueta)}
	{alpha}ANOTAR_EVENTO(1,1,alpha,grados,enx,200,200,200,200,200, CHR(224));




	trazo(0,v[1]);					{v del generador}
	trazo(2,V[1]-V[2]);        {v del tiristor}
	trazo(1,v[2]);					{v de carga}

	{mostrar pulso de disparo}
	if disparo then trazo(4,-100) else trazo(4,-300);


	{El muestreo para calculo del espectro de frecuencia se hace al final}
	mem1.tic(t,v[2]);

	if t >= tfinal then
	begin
	{llegamos al final de la simulacin}
		for k:= 0 to 4 do
		begin
			borrecanal(k,0);
			desligue(k);
			borde;
			if k<>3 then	{canal 3 es el de armonicos sin grid ni linea0}
			begin
				grid;
				linea0;
			end;
		end;

		{canal 3}
		mem1.fft;								{calcular FFT de mem1}
		UD:=mem1.ValorMedio;					{valor medio de uD}
		U1:= sqrt(mem1.PotenciaArmonica(1));
		PlotMem_EspectroUnU1Bar_0_25(3,mem1);	{mostrar armonicos calculados de mem1}
		gridx:=grillax;						{forzar grilla al valor por defecto}
		gridy:=grillay;						{forzar grilla al valor por defecto}

		{nueva simulacin}
		t:= t-tfinal+tinicial;

		mem1.RetrocesoBarrido(tfinal-tinicial);{preparar nueva carga de datos}

	end;


end;


(*********************************************************************
 Principal
*********************************************************************)
BEGIN
	quita:=5;		{peso de separacin entre pantallas}

	UD:=91.0;			{valor "inicial"}
	U1:=105.8;			{valor "inicial"}

	{decimos a simeep donde estan nuestras funciones}
	FilUSR.Cirt1:=Cirt1;
	FilUSR.Cirt2:=Cirt2;
	FilUSR.FuncsR:=FuncsR;
	FilUSR.Inicio:=Inicio;
	FilUSR.Pesca:=Pesca;
	FilUSR.Fin:=Fin;

	mem1:= TMemoria.Create_init(Periodo/Npuntos,Npuntos,0,0);		{inicializacin de memoria de datos}


	{definicin de controles}
	UCntrl.init;

	UCntrl.insertdisplay(new(PCtrlVerTexto, init
	{ rnglon, texto1  ,            texto2,             texto3}
	(	0,'Rectificador de media','onda con tiristor.','')));

	UCntrl.insertdisplay(new(PCtrlVer2Var,init
	{ renglon, nombre1, uni1,deci1,var1 ,nombre2, uni2,deci2,var2}
	(       7, 'Ud',  'V',  1  ,Ud ,'U1', 'Vef',  1  ,U1)));

	UCntrl.insert(new(PCtrlBotonVar,Init
	{renglon, nombre,   unidad,decimales, min, max,   def, inc,    var}
	(  8, ' Disparo',  ''      ,0,   0,  170,  30, 2, alpha)));

	paseo;												  {simulacin}

	mem1.Free;											  {liberacin de memoria de datos muestreados}

end.
